# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_observeinfrareddistance.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_observeinfrareddistance.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\025com.ubtechinc.codemao',
    serialized_pb=b'\n%codemao_observeinfrareddistance.proto\"M\n\x1eObserveInfraredDistanceRequest\x12\x13\n\x0bisSubscribe\x18\x01 \x01(\x08\x12\x16\n\x0esamplingPeriod\x18\x02 \x01(\x05\"3\n\x1fObserveInfraredDistanceResponse\x12\x10\n\x08\x64istance\x18\x01 \x01(\x05\x42\x17\n\x15\x63om.ubtechinc.codemaob\x06proto3'
)

_OBSERVEINFRAREDDISTANCEREQUEST = _descriptor.Descriptor(
    name='ObserveInfraredDistanceRequest',
    full_name='ObserveInfraredDistanceRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='isSubscribe', full_name='ObserveInfraredDistanceRequest.isSubscribe', index=0,
            number=1, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='samplingPeriod', full_name='ObserveInfraredDistanceRequest.samplingPeriod', index=1,
            number=2, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=41,
    serialized_end=118,
)

_OBSERVEINFRAREDDISTANCERESPONSE = _descriptor.Descriptor(
    name='ObserveInfraredDistanceResponse',
    full_name='ObserveInfraredDistanceResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='distance', full_name='ObserveInfraredDistanceResponse.distance', index=0,
            number=1, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=120,
    serialized_end=171,
)

DESCRIPTOR.message_types_by_name['ObserveInfraredDistanceRequest'] = _OBSERVEINFRAREDDISTANCEREQUEST
DESCRIPTOR.message_types_by_name['ObserveInfraredDistanceResponse'] = _OBSERVEINFRAREDDISTANCERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObserveInfraredDistanceRequest = _reflection.GeneratedProtocolMessageType('ObserveInfraredDistanceRequest',
                                                                          (_message.Message,), {
                                                                              'DESCRIPTOR': _OBSERVEINFRAREDDISTANCEREQUEST,
                                                                              '__module__': 'codemao_observeinfrareddistance_pb2'
                                                                              # @@protoc_insertion_point(class_scope:ObserveInfraredDistanceRequest)
                                                                          })
_sym_db.RegisterMessage(ObserveInfraredDistanceRequest)

ObserveInfraredDistanceResponse = _reflection.GeneratedProtocolMessageType('ObserveInfraredDistanceResponse',
                                                                           (_message.Message,), {
                                                                               'DESCRIPTOR': _OBSERVEINFRAREDDISTANCERESPONSE,
                                                                               '__module__': 'codemao_observeinfrareddistance_pb2'
                                                                               # @@protoc_insertion_point(class_scope:ObserveInfraredDistanceResponse)
                                                                           })
_sym_db.RegisterMessage(ObserveInfraredDistanceResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
