# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_observebutterystatus.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_observebutterystatus.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\025com.ubtechinc.codemao',
    serialized_pb=b'\n\"codemao_observebutterystatus.proto\"2\n\x1bObserveButteryStatusRequest\x12\x13\n\x0bisSubscribe\x18\x01 \x01(\x08\"=\n\x1cObserveButteryStatusResponse\x12\r\n\x05level\x18\x01 \x01(\x05\x12\x0e\n\x06status\x18\x02 \x01(\x05\x42\x17\n\x15\x63om.ubtechinc.codemaob\x06proto3'
)

_OBSERVEBUTTERYSTATUSREQUEST = _descriptor.Descriptor(
    name='ObserveButteryStatusRequest',
    full_name='ObserveButteryStatusRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='isSubscribe', full_name='ObserveButteryStatusRequest.isSubscribe', index=0,
            number=1, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=38,
    serialized_end=88,
)

_OBSERVEBUTTERYSTATUSRESPONSE = _descriptor.Descriptor(
    name='ObserveButteryStatusResponse',
    full_name='ObserveButteryStatusResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='level', full_name='ObserveButteryStatusResponse.level', index=0,
            number=1, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='status', full_name='ObserveButteryStatusResponse.status', index=1,
            number=2, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=90,
    serialized_end=151,
)

DESCRIPTOR.message_types_by_name['ObserveButteryStatusRequest'] = _OBSERVEBUTTERYSTATUSREQUEST
DESCRIPTOR.message_types_by_name['ObserveButteryStatusResponse'] = _OBSERVEBUTTERYSTATUSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObserveButteryStatusRequest = _reflection.GeneratedProtocolMessageType('ObserveButteryStatusRequest',
                                                                       (_message.Message,), {
                                                                           'DESCRIPTOR': _OBSERVEBUTTERYSTATUSREQUEST,
                                                                           '__module__': 'codemao_observebutterystatus_pb2'
                                                                           # @@protoc_insertion_point(class_scope:ObserveButteryStatusRequest)
                                                                       })
_sym_db.RegisterMessage(ObserveButteryStatusRequest)

ObserveButteryStatusResponse = _reflection.GeneratedProtocolMessageType('ObserveButteryStatusResponse',
                                                                        (_message.Message,), {
                                                                            'DESCRIPTOR': _OBSERVEBUTTERYSTATUSRESPONSE,
                                                                            '__module__': 'codemao_observebutterystatus_pb2'
                                                                            # @@protoc_insertion_point(class_scope:ObserveButteryStatusResponse)
                                                                        })
_sym_db.RegisterMessage(ObserveButteryStatusResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
