# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_controlrunningprogram.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_controlrunningprogram.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\025com.ubtechinc.codemao',
    serialized_pb=b'\n#codemao_controlrunningprogram.proto\"/\n\x1c\x43ontrolRunningProgramRequest\x12\x0f\n\x07isStart\x18\x01 \x01(\x08\"F\n\x1d\x43ontrolRunningProgramResponse\x12\x11\n\tisSuccess\x18\x01 \x01(\x08\x12\x12\n\nresultCode\x18\x02 \x01(\x05\x42\x17\n\x15\x63om.ubtechinc.codemaob\x06proto3'
)

_CONTROLRUNNINGPROGRAMREQUEST = _descriptor.Descriptor(
    name='ControlRunningProgramRequest',
    full_name='ControlRunningProgramRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='isStart', full_name='ControlRunningProgramRequest.isStart', index=0,
            number=1, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=39,
    serialized_end=86,
)

_CONTROLRUNNINGPROGRAMRESPONSE = _descriptor.Descriptor(
    name='ControlRunningProgramResponse',
    full_name='ControlRunningProgramResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='isSuccess', full_name='ControlRunningProgramResponse.isSuccess', index=0,
            number=1, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='resultCode', full_name='ControlRunningProgramResponse.resultCode', index=1,
            number=2, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=88,
    serialized_end=158,
)

DESCRIPTOR.message_types_by_name['ControlRunningProgramRequest'] = _CONTROLRUNNINGPROGRAMREQUEST
DESCRIPTOR.message_types_by_name['ControlRunningProgramResponse'] = _CONTROLRUNNINGPROGRAMRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ControlRunningProgramRequest = _reflection.GeneratedProtocolMessageType('ControlRunningProgramRequest',
                                                                        (_message.Message,), {
                                                                            'DESCRIPTOR': _CONTROLRUNNINGPROGRAMREQUEST,
                                                                            '__module__': 'codemao_controlrunningprogram_pb2'
                                                                            # @@protoc_insertion_point(class_scope:ControlRunningProgramRequest)
                                                                        })
_sym_db.RegisterMessage(ControlRunningProgramRequest)

ControlRunningProgramResponse = _reflection.GeneratedProtocolMessageType('ControlRunningProgramResponse',
                                                                         (_message.Message,), {
                                                                             'DESCRIPTOR': _CONTROLRUNNINGPROGRAMRESPONSE,
                                                                             '__module__': 'codemao_controlrunningprogram_pb2'
                                                                             # @@protoc_insertion_point(class_scope:ControlRunningProgramResponse)
                                                                         })
_sym_db.RegisterMessage(ControlRunningProgramResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
