# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_getrobotexpression.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from ..pb2 import codemao_expressioninfo_pb2 as codemao__expressioninfo__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_getrobotexpression.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\025com.ubtechinc.codemao',
    serialized_pb=b'\n codemao_getrobotexpression.proto\x1a\x1c\x63odemao_expressioninfo.proto\"\x1b\n\x19GetRobotExpressionRequest\"J\n\x1aGetRobotExpressionResponse\x12,\n\x0b\x65xpressList\x18\x01 \x03(\x0b\x32\x17.ExpressionInfoResponseB\x17\n\x15\x63om.ubtechinc.codemaob\x06proto3'
    ,
    dependencies=[codemao__expressioninfo__pb2.DESCRIPTOR, ])

_GETROBOTEXPRESSIONREQUEST = _descriptor.Descriptor(
    name='GetRobotExpressionRequest',
    full_name='GetRobotExpressionRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=66,
    serialized_end=93,
)

_GETROBOTEXPRESSIONRESPONSE = _descriptor.Descriptor(
    name='GetRobotExpressionResponse',
    full_name='GetRobotExpressionResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='expressList', full_name='GetRobotExpressionResponse.expressList', index=0,
            number=1, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=95,
    serialized_end=169,
)

_GETROBOTEXPRESSIONRESPONSE.fields_by_name[
    'expressList'].message_type = codemao__expressioninfo__pb2._EXPRESSIONINFORESPONSE
DESCRIPTOR.message_types_by_name['GetRobotExpressionRequest'] = _GETROBOTEXPRESSIONREQUEST
DESCRIPTOR.message_types_by_name['GetRobotExpressionResponse'] = _GETROBOTEXPRESSIONRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetRobotExpressionRequest = _reflection.GeneratedProtocolMessageType('GetRobotExpressionRequest', (_message.Message,), {
    'DESCRIPTOR': _GETROBOTEXPRESSIONREQUEST,
    '__module__': 'codemao_getrobotexpression_pb2'
    # @@protoc_insertion_point(class_scope:GetRobotExpressionRequest)
})
_sym_db.RegisterMessage(GetRobotExpressionRequest)

GetRobotExpressionResponse = _reflection.GeneratedProtocolMessageType('GetRobotExpressionResponse', (_message.Message,),
                                                                      {
                                                                          'DESCRIPTOR': _GETROBOTEXPRESSIONRESPONSE,
                                                                          '__module__': 'codemao_getrobotexpression_pb2'
                                                                          # @@protoc_insertion_point(class_scope:GetRobotExpressionResponse)
                                                                      })
_sym_db.RegisterMessage(GetRobotExpressionResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
