# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: programme_facetrack.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from ..pb2 import codemao_faceinfo_pb2 as codemao__faceinfo__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='programme_facetrack.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\035com.ubtechinc.alpha.programme',
    serialized_pb=b'\n\x19programme_facetrack.proto\x1a\x16\x63odemao_faceinfo.proto\"8\n\x10\x46\x61\x63\x65TrackRequest\x12\x0f\n\x07timeout\x18\x01 \x01(\x05\x12\x13\n\x0bisNeedTrack\x18\x02 \x01(\x08\"`\n\x11\x46\x61\x63\x65TrackResponse\x12$\n\tfaceInfos\x18\x01 \x03(\x0b\x32\x11.FaceInfoResponse\x12\x11\n\tisSuccess\x18\x02 \x01(\x08\x12\x12\n\nresultCode\x18\x03 \x01(\x05\x42\x1f\n\x1d\x63om.ubtechinc.alpha.programmeb\x06proto3'
    ,
    dependencies=[codemao__faceinfo__pb2.DESCRIPTOR, ])

_FACETRACKREQUEST = _descriptor.Descriptor(
    name='FaceTrackRequest',
    full_name='FaceTrackRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='timeout', full_name='FaceTrackRequest.timeout', index=0,
            number=1, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='isNeedTrack', full_name='FaceTrackRequest.isNeedTrack', index=1,
            number=2, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=53,
    serialized_end=109,
)

_FACETRACKRESPONSE = _descriptor.Descriptor(
    name='FaceTrackResponse',
    full_name='FaceTrackResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='faceInfos', full_name='FaceTrackResponse.faceInfos', index=0,
            number=1, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='isSuccess', full_name='FaceTrackResponse.isSuccess', index=1,
            number=2, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='resultCode', full_name='FaceTrackResponse.resultCode', index=2,
            number=3, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=111,
    serialized_end=207,
)

_FACETRACKRESPONSE.fields_by_name['faceInfos'].message_type = codemao__faceinfo__pb2._FACEINFORESPONSE
DESCRIPTOR.message_types_by_name['FaceTrackRequest'] = _FACETRACKREQUEST
DESCRIPTOR.message_types_by_name['FaceTrackResponse'] = _FACETRACKRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FaceTrackRequest = _reflection.GeneratedProtocolMessageType('FaceTrackRequest', (_message.Message,), {
    'DESCRIPTOR': _FACETRACKREQUEST,
    '__module__': 'programme_facetrack_pb2'
    # @@protoc_insertion_point(class_scope:FaceTrackRequest)
})
_sym_db.RegisterMessage(FaceTrackRequest)

FaceTrackResponse = _reflection.GeneratedProtocolMessageType('FaceTrackResponse', (_message.Message,), {
    'DESCRIPTOR': _FACETRACKRESPONSE,
    '__module__': 'programme_facetrack_pb2'
    # @@protoc_insertion_point(class_scope:FaceTrackResponse)
})
_sym_db.RegisterMessage(FaceTrackResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
