# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_facerecognise.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from ..pb2 import codemao_faceinfo_pb2 as codemao__faceinfo__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_facerecognise.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\025com.ubtechinc.codemao',
    serialized_pb=b'\n\x1b\x63odemao_facerecognise.proto\x1a\x16\x63odemao_faceinfo.proto\"\'\n\x14\x46\x61\x63\x65RecogniseRequest\x12\x0f\n\x07timeout\x18\x01 \x01(\x05\"w\n\x15\x46\x61\x63\x65RecogniseResponse\x12$\n\tfaceInfos\x18\x01 \x03(\x0b\x32\x11.FaceInfoResponse\x12\x11\n\tisSuccess\x18\x02 \x01(\x08\x12\x12\n\nresultCode\x18\x03 \x01(\x05\x12\x11\n\tcommandId\x18\x04 \x01(\tB\x17\n\x15\x63om.ubtechinc.codemaob\x06proto3'
    ,
    dependencies=[codemao__faceinfo__pb2.DESCRIPTOR, ])

_FACERECOGNISEREQUEST = _descriptor.Descriptor(
    name='FaceRecogniseRequest',
    full_name='FaceRecogniseRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='timeout', full_name='FaceRecogniseRequest.timeout', index=0,
            number=1, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=55,
    serialized_end=94,
)

_FACERECOGNISERESPONSE = _descriptor.Descriptor(
    name='FaceRecogniseResponse',
    full_name='FaceRecogniseResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='faceInfos', full_name='FaceRecogniseResponse.faceInfos', index=0,
            number=1, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='isSuccess', full_name='FaceRecogniseResponse.isSuccess', index=1,
            number=2, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='resultCode', full_name='FaceRecogniseResponse.resultCode', index=2,
            number=3, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='commandId', full_name='FaceRecogniseResponse.commandId', index=3,
            number=4, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=96,
    serialized_end=215,
)

_FACERECOGNISERESPONSE.fields_by_name['faceInfos'].message_type = codemao__faceinfo__pb2._FACEINFORESPONSE
DESCRIPTOR.message_types_by_name['FaceRecogniseRequest'] = _FACERECOGNISEREQUEST
DESCRIPTOR.message_types_by_name['FaceRecogniseResponse'] = _FACERECOGNISERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FaceRecogniseRequest = _reflection.GeneratedProtocolMessageType('FaceRecogniseRequest', (_message.Message,), {
    'DESCRIPTOR': _FACERECOGNISEREQUEST,
    '__module__': 'codemao_facerecognise_pb2'
    # @@protoc_insertion_point(class_scope:FaceRecogniseRequest)
})
_sym_db.RegisterMessage(FaceRecogniseRequest)

FaceRecogniseResponse = _reflection.GeneratedProtocolMessageType('FaceRecogniseResponse', (_message.Message,), {
    'DESCRIPTOR': _FACERECOGNISERESPONSE,
    '__module__': 'codemao_facerecognise_pb2'
    # @@protoc_insertion_point(class_scope:FaceRecogniseResponse)
})
_sym_db.RegisterMessage(FaceRecogniseResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
