# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_translate.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from ..pb2 import cloudtranslate_pb2 as cloudtranslate__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_translate.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\023com.ubtechinc.alpha',
    serialized_pb=b'\n\x17\x63odemao_translate.proto\x1a\x14\x63loudtranslate.proto\"1\n\x10TranslateRequest\x12\x1d\n\ttranslate\x18\x01 \x01(\x0b\x32\n.Translate\":\n\x11TranslateResponse\x12\x11\n\tisSuccess\x18\x01 \x01(\x08\x12\x12\n\nresultCode\x18\x02 \x01(\x05\x42\x15\n\x13\x63om.ubtechinc.alphab\x06proto3'
    ,
    dependencies=[cloudtranslate__pb2.DESCRIPTOR, ])

_TRANSLATEREQUEST = _descriptor.Descriptor(
    name='TranslateRequest',
    full_name='TranslateRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='translate', full_name='TranslateRequest.translate', index=0,
            number=1, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=49,
    serialized_end=98,
)

_TRANSLATERESPONSE = _descriptor.Descriptor(
    name='TranslateResponse',
    full_name='TranslateResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='isSuccess', full_name='TranslateResponse.isSuccess', index=0,
            number=1, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='resultCode', full_name='TranslateResponse.resultCode', index=1,
            number=2, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=100,
    serialized_end=158,
)

_TRANSLATEREQUEST.fields_by_name['translate'].message_type = cloudtranslate__pb2._TRANSLATE
DESCRIPTOR.message_types_by_name['TranslateRequest'] = _TRANSLATEREQUEST
DESCRIPTOR.message_types_by_name['TranslateResponse'] = _TRANSLATERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TranslateRequest = _reflection.GeneratedProtocolMessageType('TranslateRequest', (_message.Message,), {
    'DESCRIPTOR': _TRANSLATEREQUEST,
    '__module__': 'codemao_translate_pb2'
    # @@protoc_insertion_point(class_scope:TranslateRequest)
})
_sym_db.RegisterMessage(TranslateRequest)

TranslateResponse = _reflection.GeneratedProtocolMessageType('TranslateResponse', (_message.Message,), {
    'DESCRIPTOR': _TRANSLATERESPONSE,
    '__module__': 'codemao_translate_pb2'
    # @@protoc_insertion_point(class_scope:TranslateResponse)
})
_sym_db.RegisterMessage(TranslateResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
