# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cloudstorageurls.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import enum_type_wrapper

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='cloudstorageurls.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\023com.ubtechinc.alpha',
    serialized_pb=b'\n\x16\x63loudstorageurls.proto\"3\n\x0c\x43loudStorage\x12\x0b\n\x03url\x18\x01 \x03(\t\x12\x16\n\x04type\x18\x02 \x01(\x0e\x32\x08.UrlType*g\n\x07UrlType\x12\x11\n\rQINIU_PRIVATE\x10\x00\x12\x12\n\x0e\x41LIYUN_PRIVATE\x10\x01\x12\x0e\n\nNET_PUBLIC\x10\x02\x12\x10\n\x0cPRESET_LOCAL\x10\x03\x12\x13\n\x0f\x43USTOMIZE_LOCAL\x10\x04\x42\x15\n\x13\x63om.ubtechinc.alphab\x06proto3'
)

_URLTYPE = _descriptor.EnumDescriptor(
    name='UrlType',
    full_name='UrlType',
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name='QINIU_PRIVATE', index=0, number=0,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='ALIYUN_PRIVATE', index=1, number=1,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='NET_PUBLIC', index=2, number=2,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='PRESET_LOCAL', index=3, number=3,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='CUSTOMIZE_LOCAL', index=4, number=4,
            serialized_options=None,
            type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=79,
    serialized_end=182,
)
_sym_db.RegisterEnumDescriptor(_URLTYPE)

UrlType = enum_type_wrapper.EnumTypeWrapper(_URLTYPE)
QINIU_PRIVATE = 0
ALIYUN_PRIVATE = 1
NET_PUBLIC = 2
PRESET_LOCAL = 3
CUSTOMIZE_LOCAL = 4

_CLOUDSTORAGE = _descriptor.Descriptor(
    name='CloudStorage',
    full_name='CloudStorage',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='url', full_name='CloudStorage.url', index=0,
            number=1, type=9, cpp_type=9, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='type', full_name='CloudStorage.type', index=1,
            number=2, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=26,
    serialized_end=77,
)

_CLOUDSTORAGE.fields_by_name['type'].enum_type = _URLTYPE
DESCRIPTOR.message_types_by_name['CloudStorage'] = _CLOUDSTORAGE
DESCRIPTOR.enum_types_by_name['UrlType'] = _URLTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CloudStorage = _reflection.GeneratedProtocolMessageType('CloudStorage', (_message.Message,), {
    'DESCRIPTOR': _CLOUDSTORAGE,
    '__module__': 'cloudstorageurls_pb2'
    # @@protoc_insertion_point(class_scope:CloudStorage)
})
_sym_db.RegisterMessage(CloudStorage)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
