# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cloudtranslate.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import enum_type_wrapper

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='cloudtranslate.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\023com.ubtechinc.alpha',
    serialized_pb=b'\n\x14\x63loudtranslate.proto\"\x8f\x01\n\tTranslate\x12\r\n\x05query\x18\x01 \x01(\t\x12\x0e\n\x06prefix\x18\x02 \x01(\t\x12\x0e\n\x06suffix\x18\x03 \x01(\t\x12\x1b\n\x08platform\x18\x04 \x01(\x0e\x32\t.Platform\x12\x1b\n\x04\x66rom\x18\x05 \x01(\x0e\x32\r.LanguageType\x12\x19\n\x02to\x18\x06 \x01(\x0e\x32\r.LanguageType*\xde\x02\n\x0cLanguageType\x12\x06\n\x02\x43N\x10\x00\x12\x06\n\x02\x45N\x10\x01\x12\x06\n\x02TW\x10\x02\x12\x06\n\x02\x46R\x10\x03\x12\x06\n\x02\x44\x45\x10\x04\x12\x06\n\x02\x45L\x10\x05\x12\x06\n\x02JA\x10\x06\x12\x06\n\x02KO\x10\x07\x12\x06\n\x02RU\x10\x08\x12\x06\n\x02\x43S\x10\t\x12\x06\n\x02\x44\x41\x10\n\x12\x06\n\x02NL\x10\x0b\x12\x06\n\x02\x46I\x10\x0c\x12\x06\n\x02\x42N\x10\r\x12\x06\n\x02\x41\x46\x10\x0e\x12\x06\n\x02\x45U\x10\x0f\x12\x06\n\x02\x42G\x10\x10\x12\x06\n\x02\x42\x45\x10\x11\x12\x06\n\x02\x45O\x10\x12\x12\x06\n\x02ID\x10\x13\x12\x06\n\x02IT\x10\x14\x12\x06\n\x02GA\x10\x15\x12\x06\n\x02KY\x10\x16\x12\x06\n\x02LA\x10\x17\x12\x06\n\x02LO\x10\x18\x12\x06\n\x02MS\x10\x19\x12\x06\n\x02NO\x10\x1a\x12\x06\n\x02\x46\x41\x10\x1b\x12\x06\n\x02PT\x10\x1c\x12\x06\n\x02SR\x10\x1d\x12\x06\n\x02SO\x10\x1e\x12\x06\n\x02UZ\x10\x1f\x12\x06\n\x02ZU\x10 \x12\x06\n\x02YO\x10!\x12\x06\n\x02VI\x10\"\x12\x06\n\x02TH\x10#\x12\x06\n\x02TG\x10$\x12\x06\n\x02SK\x10%\x12\x06\n\x02\x41R\x10&\x12\x06\n\x02MN\x10\'\x12\x06\n\x02MY\x10(\x12\x06\n\x02TL\x10)*.\n\x08Platform\x12\t\n\x05\x42\x41IDU\x10\x00\x12\x0b\n\x07TENCENT\x10\x01\x12\n\n\x06GOOGLE\x10\x02\x42\x15\n\x13\x63om.ubtechinc.alphab\x06proto3'
)

_LANGUAGETYPE = _descriptor.EnumDescriptor(
    name='LanguageType',
    full_name='LanguageType',
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name='CN', index=0, number=0,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='EN', index=1, number=1,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='TW', index=2, number=2,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='FR', index=3, number=3,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='DE', index=4, number=4,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='EL', index=5, number=5,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='JA', index=6, number=6,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='KO', index=7, number=7,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='RU', index=8, number=8,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='CS', index=9, number=9,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='DA', index=10, number=10,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='NL', index=11, number=11,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='FI', index=12, number=12,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='BN', index=13, number=13,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='AF', index=14, number=14,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='EU', index=15, number=15,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='BG', index=16, number=16,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='BE', index=17, number=17,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='EO', index=18, number=18,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='ID', index=19, number=19,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='IT', index=20, number=20,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='GA', index=21, number=21,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='KY', index=22, number=22,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='LA', index=23, number=23,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='LO', index=24, number=24,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='MS', index=25, number=25,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='NO', index=26, number=26,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='FA', index=27, number=27,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='PT', index=28, number=28,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='SR', index=29, number=29,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='SO', index=30, number=30,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='UZ', index=31, number=31,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='ZU', index=32, number=32,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='YO', index=33, number=33,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='VI', index=34, number=34,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='TH', index=35, number=35,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='TG', index=36, number=36,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='SK', index=37, number=37,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='AR', index=38, number=38,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='MN', index=39, number=39,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='MY', index=40, number=40,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='TL', index=41, number=41,
            serialized_options=None,
            type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=171,
    serialized_end=521,
)
_sym_db.RegisterEnumDescriptor(_LANGUAGETYPE)

LanguageType = enum_type_wrapper.EnumTypeWrapper(_LANGUAGETYPE)
_PLATFORM = _descriptor.EnumDescriptor(
    name='Platform',
    full_name='Platform',
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name='BAIDU', index=0, number=0,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='TENCENT', index=1, number=1,
            serialized_options=None,
            type=None),
        _descriptor.EnumValueDescriptor(
            name='GOOGLE', index=2, number=2,
            serialized_options=None,
            type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=523,
    serialized_end=569,
)
_sym_db.RegisterEnumDescriptor(_PLATFORM)

Platform = enum_type_wrapper.EnumTypeWrapper(_PLATFORM)
CN = 0
EN = 1
TW = 2
FR = 3
DE = 4
EL = 5
JA = 6
KO = 7
RU = 8
CS = 9
DA = 10
NL = 11
FI = 12
BN = 13
AF = 14
EU = 15
BG = 16
BE = 17
EO = 18
ID = 19
IT = 20
GA = 21
KY = 22
LA = 23
LO = 24
MS = 25
NO = 26
FA = 27
PT = 28
SR = 29
SO = 30
UZ = 31
ZU = 32
YO = 33
VI = 34
TH = 35
TG = 36
SK = 37
AR = 38
MN = 39
MY = 40
TL = 41
BAIDU = 0
TENCENT = 1
GOOGLE = 2

_TRANSLATE = _descriptor.Descriptor(
    name='Translate',
    full_name='Translate',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='query', full_name='Translate.query', index=0,
            number=1, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='prefix', full_name='Translate.prefix', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='suffix', full_name='Translate.suffix', index=2,
            number=3, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='platform', full_name='Translate.platform', index=3,
            number=4, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='from', full_name='Translate.from', index=4,
            number=5, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='to', full_name='Translate.to', index=5,
            number=6, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=25,
    serialized_end=168,
)

_TRANSLATE.fields_by_name['platform'].enum_type = _PLATFORM
_TRANSLATE.fields_by_name['from'].enum_type = _LANGUAGETYPE
_TRANSLATE.fields_by_name['to'].enum_type = _LANGUAGETYPE
DESCRIPTOR.message_types_by_name['Translate'] = _TRANSLATE
DESCRIPTOR.enum_types_by_name['LanguageType'] = _LANGUAGETYPE
DESCRIPTOR.enum_types_by_name['Platform'] = _PLATFORM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Translate = _reflection.GeneratedProtocolMessageType('Translate', (_message.Message,), {
    'DESCRIPTOR': _TRANSLATE,
    '__module__': 'cloudtranslate_pb2'
    # @@protoc_insertion_point(class_scope:Translate)
})
_sym_db.RegisterMessage(Translate)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
