# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_playonlinemusic.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from ..pb2 import cloudtranslate_pb2 as cloudtranslate__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_playonlinemusic.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\023com.ubtechinc.alpha',
    serialized_pb=b'\n\x1d\x63odemao_playonlinemusic.proto\x1a\x14\x63loudtranslate.proto\"9\n\x0cMusicRequest\x12\x1b\n\x08platform\x18\x01 \x01(\x0e\x32\t.Platform\x12\x0c\n\x04name\x18\x02 \x01(\t\"6\n\rMusicResponse\x12\x11\n\tisSuccess\x18\x01 \x01(\x08\x12\x12\n\nresultCode\x18\x02 \x01(\x05\x42\x15\n\x13\x63om.ubtechinc.alphab\x06proto3'
    ,
    dependencies=[cloudtranslate__pb2.DESCRIPTOR, ])

_MUSICREQUEST = _descriptor.Descriptor(
    name='MusicRequest',
    full_name='MusicRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='platform', full_name='MusicRequest.platform', index=0,
            number=1, type=14, cpp_type=8, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='name', full_name='MusicRequest.name', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=b"".decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=55,
    serialized_end=112,
)

_MUSICRESPONSE = _descriptor.Descriptor(
    name='MusicResponse',
    full_name='MusicResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='isSuccess', full_name='MusicResponse.isSuccess', index=0,
            number=1, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='resultCode', full_name='MusicResponse.resultCode', index=1,
            number=2, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=114,
    serialized_end=168,
)

_MUSICREQUEST.fields_by_name['platform'].enum_type = cloudtranslate__pb2._PLATFORM
DESCRIPTOR.message_types_by_name['MusicRequest'] = _MUSICREQUEST
DESCRIPTOR.message_types_by_name['MusicResponse'] = _MUSICRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MusicRequest = _reflection.GeneratedProtocolMessageType('MusicRequest', (_message.Message,), {
    'DESCRIPTOR': _MUSICREQUEST,
    '__module__': 'codemao_playonlinemusic_pb2'
    # @@protoc_insertion_point(class_scope:MusicRequest)
})
_sym_db.RegisterMessage(MusicRequest)

MusicResponse = _reflection.GeneratedProtocolMessageType('MusicResponse', (_message.Message,), {
    'DESCRIPTOR': _MUSICRESPONSE,
    '__module__': 'codemao_playonlinemusic_pb2'
    # @@protoc_insertion_point(class_scope:MusicResponse)
})
_sym_db.RegisterMessage(MusicResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
