# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: codemao_observevolumekeypress.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='codemao_observevolumekeypress.proto',
    package='',
    syntax='proto3',
    serialized_options=b'\n\025com.ubtechinc.codemao',
    serialized_pb=b'\n#codemao_observevolumekeypress.proto\"3\n\x1cObserveVolumeKeyPressRequest\x12\x13\n\x0bisSubscribe\x18\x01 \x01(\x08\"-\n\x1dObserveVolumeKeyPressResponse\x12\x0c\n\x04type\x18\x01 \x01(\x05\x42\x17\n\x15\x63om.ubtechinc.codemaob\x06proto3'
)

_OBSERVEVOLUMEKEYPRESSREQUEST = _descriptor.Descriptor(
    name='ObserveVolumeKeyPressRequest',
    full_name='ObserveVolumeKeyPressRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='isSubscribe', full_name='ObserveVolumeKeyPressRequest.isSubscribe', index=0,
            number=1, type=8, cpp_type=7, label=1,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=39,
    serialized_end=90,
)

_OBSERVEVOLUMEKEYPRESSRESPONSE = _descriptor.Descriptor(
    name='ObserveVolumeKeyPressResponse',
    full_name='ObserveVolumeKeyPressResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='type', full_name='ObserveVolumeKeyPressResponse.type', index=0,
            number=1, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=92,
    serialized_end=137,
)

DESCRIPTOR.message_types_by_name['ObserveVolumeKeyPressRequest'] = _OBSERVEVOLUMEKEYPRESSREQUEST
DESCRIPTOR.message_types_by_name['ObserveVolumeKeyPressResponse'] = _OBSERVEVOLUMEKEYPRESSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObserveVolumeKeyPressRequest = _reflection.GeneratedProtocolMessageType('ObserveVolumeKeyPressRequest',
                                                                        (_message.Message,), {
                                                                            'DESCRIPTOR': _OBSERVEVOLUMEKEYPRESSREQUEST,
                                                                            '__module__': 'codemao_observevolumekeypress_pb2'
                                                                            # @@protoc_insertion_point(class_scope:ObserveVolumeKeyPressRequest)
                                                                        })
_sym_db.RegisterMessage(ObserveVolumeKeyPressRequest)

ObserveVolumeKeyPressResponse = _reflection.GeneratedProtocolMessageType('ObserveVolumeKeyPressResponse',
                                                                         (_message.Message,), {
                                                                             'DESCRIPTOR': _OBSERVEVOLUMEKEYPRESSRESPONSE,
                                                                             '__module__': 'codemao_observevolumekeypress_pb2'
                                                                             # @@protoc_insertion_point(class_scope:ObserveVolumeKeyPressResponse)
                                                                         })
_sym_db.RegisterMessage(ObserveVolumeKeyPressResponse)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
